//+---------------------------------------------------------------------+
//|                                            multiwebobjectworker.mq5 |
//|                                       Copyright (c) 2018, Marketeer |
//|                             https://www.mql5.com/en/users/marketeer |
//| Asynchronous Non-Blocking WebRequest Implemenation in Chart Objects |
//|                               https://www.mql5.com/ru/articles/5337 |
//+---------------------------------------------------------------------+
#property copyright   "Copyright (c) 2018, Marketeer"
#property link        "https://www.mql5.com/en/users/marketeer"
#property description "WebRequest execution in chart objects created ad hoc\n"

#include <multiwebobjectworker.mqh>

class WebWorkerScriptHelper: public WebWorkerScript
{
  public:
    WebWorkerScriptHelper(const long id): WebWorkerScript(id)
    {
    }

    void receive(const string source, const uchar &result[], const string &result_headers)
    {
      ResourceMediator mediator(allocate());
      mediator.packResponse(source, result, result_headers);
    }
};

WebWorkerScriptHelper wws(ChartID());

void OnInit()
{
  Print(ChartID(), ": ", __FUNCTION__);
  startWebRequest(MultiWebScriptPath);
  /*  // doesn't work in objects
  EventSetTimer(1);
  */
}

void OnDeinit(const int reason)
{
  Print(ChartID(), ": ", __FUNCTION__);
}

/* // doesn't work in objects
void OnTimer()
{
  Print(ChartID(), __FUNCTION__);
  EventKillTimer();
  startWebRequest(MultiWebScriptPath);
}
*/

/* // doesn't work in objects
void OnTick()
{
  startWebRequest(MultiWebScriptPath);
}
*/


void startWebRequest(const string resname)
{
  Print(ChartID(), ": Reading request ", resname);
  
  long returnChartID = WebWorkerScript::getChartIDFromResource(resname);
  
  if(returnChartID == 0)
  {
    Print("Wrong sender ID: ", resname);
    return;
  }
  
  const RESOURCEDATA<uchar> resource(resname);
  ResourceMediator mediator(&resource);

  string method, url, headers;
  int timeout;
  uchar body[];

  mediator.unpackRequest(method, url, headers, timeout, body);

  Print(ChartID(), ": ", method, " ", url, " ", headers, " ", timeout, " ", CharArrayToString(body));
  Comment(method, " ", url, "\n", headers);

  uint startTime = GetTickCount();
  
  wws.release();
  
  char result[];
  string result_headers;
  
  int code = WebRequest(method, url, headers, timeout, body, result, result_headers);
  if(code != -1)
  {
    // create resource with results to pass back to the client via custom event
    wws.receive(resname, result, result_headers);
    EventChartCustom(returnChartID, TO_MSG(MSG_DONE), ChartID(), (double)code, wws.getFullName());
    Print(ChartID(), ": ", "Done in ", (GetTickCount() - startTime), "ms");
  }
  else
  {
    EventChartCustom(returnChartID, TO_MSG(MSG_ERROR), GetLastError(), 0.0, resname);
    Print(ChartID(), ": ", "Failed with code ", GetLastError());
  }
  
  // expert keeps running until client reads results and closes the object
}
